#!/bin/bash
COM_ERR=1
INSTALL_DIR=/opt/fan
SCRIPTS=scripts
CONF_FILE=fan.conf
LIB=fan.lib
source $INSTALL_DIR/$CONF_FILE
ARGS="-i2c_addr $I2C_ADDR -conf_speed $SPEED -conf_units $UNITS -conf_target $TARGET -conf_locked_detect $LOCKED_ROTOR_DETECT"
RANGE=0

pause_fan_speed_daemon()
{
	if [ $SPEED = "AUTO" ]
	then
		echo -n "Pausing Fan Speed Control Daemon"
		sudo $INSTALL_DIR/$LIB $ARGS -conf_speed 16 -restart
	
		#wait at least 2 seconds
		for i in `seq 1 10`;
   	do
     		echo -n "."
     		sleep 0.3
   	done 
		echo " "
	fi		
}

start_fan_speed_daemon()
{
	sudo $INSTALL_DIR/$LIB $ARGS -restart &
}


if [ $# -eq 0 ]
then
	echo "Please type \"fan.sh -help\" for a list of commands"
	exit 0
fi


if [ $1 = "-help" ]
then
	COM_ERR=0
	
	echo "List of commands"
	echo ""
	
	echo "-help"
	echo "-board_info"
	echo "-status"
	echo "-speed <AUTO / 0-16>"
	echo "-target <temperature>"
	echo "-i2c_find"
	echo "-i2c_change_addr <new_addr>"
	echo "-sw_rev"
	echo "-config"
	echo "-read_conf"
	echo "-fw_update <file_name>"
	echo "-clear_errors"
	echo "-restart"
	
#Read Board information
elif [ $1 = "-board_info" ]
then
	COM_ERR=0
	sudo $INSTALL_DIR/$LIB $ARGS -board_info
	exit $?
	

#Read Overall StatusBoard information
elif [ $1 = "-status" ]
then
	COM_ERR=0
	sudo $INSTALL_DIR/$LIB $ARGS -status
	exit $?

#Adjust fan speed
elif [ $1 = "-speed" ]
then
	
	if [ $# -eq 2 ]
	then
		#Check range
		RANGE=0
		if [ $2 = "AUTO" ]
		then
			RANGE=1
		elif [[ $2 =~ ^[0-9]+$ ]]
		then
			if (( 0 <= $2 && $2 <= 16))
			then
				RANGE=1;
			fi
		fi
		
		if [ $RANGE = 1 ]
		then
			sudo sed -i "/SPEED=/c\SPEED=$2" $INSTALL_DIR/$CONF_FILE
			pause_fan_speed_daemon
			sudo $INSTALL_DIR/$LIB $ARGS -conf_speed $2 -restart &
		else
			echo "ERROR: Speed($2) out of range, expecting AUTO or 0 to 16"
		   echo "For example fan.sh -speed AUTO"
		fi
	else
		echo "ERROR: Expecting another argument"
		echo "For example fan.sh -speed AUTO"
	fi
	exit $?
	
#Target temperature
elif [ $1 = "-target" ]
then
	
	if [ $# -eq 2 ]
	then
		#Check range
		RANGE=0
		if [[ $2 =~ ^[+-]?[0-9]+\.?[0-9]*$ ]]
		then
			RANGE=1
		fi
		
		if [ $RANGE = 1 ]
		then
			sudo sed -i "/TARGET=/c\TARGET=$2" $INSTALL_DIR/$CONF_FILE
			pause_fan_speed_daemon
			sudo $INSTALL_DIR/$LIB $ARGS -conf_target $2 -restart &
			echo "Target Temperature Updated to $2"
		else
		 echo "ERROR: Target Temperature($2) error, expecting a floating point number"
		 echo "For example fan.sh -target 60.5"
		fi
	else
		echo "ERROR: Expecting another argument for the target temperature"
		echo "For example fan.sh -target 60.5"
	fi
	exit $?
			
#Scans the i2c bus to find the device
#Note: I2C address inside .conf file will be updated automatically 
elif [ $1 = "-i2c_find" ]
then
	COM_ERR=0
	sudo $INSTALL_DIR/$LIB $ARGS -i2c_find
	
	tmp="$(printf '%x\n' $?)"
	
	if [ $tmp != ff ]
	then
		#Update .conf file
		echo "Updating I2C_ADDR variable inside $INSTALL_DIR/$CONF_FILE to $tmp"
		sudo sed -i "/I2C_ADDR=/c\I2C_ADDR=$tmp" $INSTALL_DIR/$CONF_FILE
	else
		echo $?
	fi


#Change I2C address
#Note: I2C address inside .conf file will be updated automatically 
elif [ $1 = "-i2c_change_addr" ]
then
	if [ $# -eq 2 ]
	then
		COM_ERR=0
		
		pause_fan_speed_daemon
		sudo $INSTALL_DIR/$LIB $ARGS -i2c_change_addr $2
		
		if [ $? -eq 0 ]
		then
			#Update .conf file
			echo "Updating I2C_ADDR variable inside $CONF_FILE to $2"
			sudo sed -i "/I2C_ADDR=/c\I2C_ADDR=$2" $INSTALL_DIR/$CONF_FILE
			sudo $INSTALL_DIR/$LIB $ARGS -i2c_addr $2 -restart &
		else
			echo "ERROR: Failed to update I2C address."
		fi
		
		exit $?
	else
		echo "ERROR: Expecting a hex number as an argument"
	fi
		
#Print software revision
elif [ $1 = "-sw_rev" ]
then
	sudo $INSTALL_DIR/$LIB $ARGS -sw_rev
	exit $?
	
elif [ $1 = "-config" ]
then
	COM_ERR=0
	echo "To configure the fan please edit the file $INSTALL_DIR/$CONF_FILE with a text editor"
	echo "For example : sudo nano $INSTALL_DIR/$CONF_FILE"
	exit $?
	
#Read the .conf file
elif [ $1 = "-read_conf" ]
then
	echo "Reading $INSTALL_DIR/$CONF_FILE"
	echo ""
	cat $INSTALL_DIR/$CONF_FILE
	exit $?


#Firmware update
elif [ $1 = "-fw_update" ]
then
	
	if [ $# -ne 2 ]
	then
		echo "Missing file name argument"
	elif [ -f $2 ]
	then
		COM_ERR=0
		
		pause_fan_speed_daemon
		sudo $INSTALL_DIR/$LIB $ARGS -fw_update $2
		
		if [ $? -eq 0 ]
		then
			start_fan_speed_daemon
		fi
	else
		echo "File doesn't exist"	
	fi
	
	exit $?


#Clear errors
elif [ $1 = "-clear_errors" ]
then
	COM_ERR=0
	sudo $INSTALL_DIR/$LIB $ARGS -clear_errors
	exit $?
	
#Loads the contents of the .conf file into the controller
#Called on Raspberry Pi power up, and can be called by the user
#after modifying the .conf file with a text editor
elif [ $1 = "-restart" ]
then
	COM_ERR=0
		
	#Don't use pause_fan_speed_daemon function becaues .conf file could have been changed
	echo -n "Pausing Fan Speed Control Daemon"
	sudo $INSTALL_DIR/$LIB $ARGS -conf_speed 16 -restart

	#wait at least 2 seconds
	for i in `seq 1 10`;
 	do
   	echo -n "."
   	sleep 0.3
 	done 
	echo " "
		
		 		
	start_fan_speed_daemon
	exit $?
else
	echo "Unknown command($1), please use -help to see valid commands"
fi



if [ $COM_ERR = "1" ]
then
	echo "Error"
fi
	
	